import Popper from 'popper.js';
import getPopperDropdownConfig from './get-popper-dropdown-config';
import getUniqueId from '../../utilities/get-unique-id';
import addClass from '../../utilities/add-class';
import removeClass from '../../utilities/remove-class';
import hasClass from '../../utilities/has-class';
import clickWasInside from '../../utilities/click-was-inside';
import getTransitionDurationFromElement from '../../utilities/get-transition-duration-from-element';
import closest from '../../utilities/closest';
import customEvent from '../../utilities/custom-event';
// TODO:
// data-offset="10,20"
// data-reference="parent"
// popper.js stuff:
// offset
// flip
// boundary
// reference
// display
export class BsDropdown {
    constructor() {
        this.show = false;
        this.keepOpen = false;
        this.offset = 0;
        this.flip = true;
        this.boundary = 'scrollParent';
        this.reference = 'toggle';
        this.display = 'dynamic';
        this.showEventName = 'show.bs.dropdown';
        this.shownEventName = 'shown.bs.dropdown';
        this.hideEventName = 'hide.bs.dropdown';
        this.hiddenEventName = 'hidden.bs.dropdown';
        this.handleDropdownClickOutside = (event) => {
            // https://gist.github.com/Restuta/e400a555ba24daa396cc
            // handleDropdownClickOutside is an arrow function so that removeEventListener will work
            if (!this.show) {
                return;
            }
            const clickWasInsideDropdown = clickWasInside(event.target, `[data-bs-id="${this.dropdownId}"]`);
            if (!clickWasInsideDropdown || !this.keepOpen) {
                document.removeEventListener('click', this.handleDropdownClickOutside);
                this.show = true;
                this.handleHideDropdown();
            }
        };
        this.handleToggleDropdownOnToggleClick = () => {
            if (this.show === true) {
                this.handleHideDropdown();
            }
            else {
                this.handleShowDropdown();
            }
        };
    }
    // @Event() show_bs_dropdown: EventEmitter;
    // @Event() shown_bs_dropdown: EventEmitter;
    // @Event() hide_bs_dropdown: EventEmitter;
    // @Event() hidden_bs_dropdown: EventEmitter;
    componentWillLoad() {
        if (hasClass(this.dropdownEl, 'show') && this.show === false) {
            this.show = true;
        }
        if (this.show === true) {
            setTimeout(() => {
                document.removeEventListener('click', this.handleDropdownClickOutside);
                document.addEventListener('click', this.handleDropdownClickOutside);
            }, 0);
        }
        this.dropdownId = getUniqueId('dropdown');
        this.dropdownEl.dataset.bsId = this.dropdownId;
        // this.dropdownEl.setAttribute('data-bs-id', this.dropdownId);
        const toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        for (let j = 0, len = toggles.length; j < len; j++) {
            toggles[j].removeEventListener('click', this.handleToggleDropdownOnToggleClick);
            toggles[j].addEventListener('click', this.handleToggleDropdownOnToggleClick);
        }
    }
    componentDidUnload() {
        document.removeEventListener('click', this.handleDropdownClickOutside);
        const toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        for (let j = 0, len = toggles.length; j < len; j++) {
            toggles[j].removeEventListener('click', this.handleToggleDropdownOnToggleClick);
        }
        if (this.popperHandle !== null) {
            this.popperHandle.destroy();
            this.popperHandle = null;
        }
    }
    initPopper(dropdownMenuEl) {
        this.inNavbar = this.detectNavbar(this.dropdownEl);
        if (!this.inNavbar) {
            const popperSettings = {
                offset: this.offset,
                flip: this.flip,
                boundary: this.boundary,
                display: this.display,
            };
            const popperConfig = getPopperDropdownConfig(this.dropdownEl, dropdownMenuEl, popperSettings);
            // If boundary is not `scrollParent`, then set position to `static`
            // to allow the menu to "escape" the scroll parent's boundaries
            // https://github.com/twbs/bootstrap/issues/24251
            if (this.boundary !== 'scrollParent') {
                addClass(this.dropdownEl, 'position-static');
            }
            let referenceElement = this.dropdownEl;
            if (this.reference === 'parent') {
                referenceElement = this.dropdownEl.parentNode;
            }
            else if (this.reference instanceof Element) {
                referenceElement = this.reference;
            }
            this.popperHandle = new Popper(referenceElement, dropdownMenuEl, popperConfig);
        }
    }
    detectNavbar(el) {
        return closest(el, '.navbar') !== null;
    }
    handleShowDropdown() {
        this.show = true;
        // this.show_bs_dropdown.emit(event);
        customEvent(this.dropdownEl, this.showEventName);
        const dropdownMenuEl = this.dropdownEl.querySelector('.dropdown-menu');
        const toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        addClass(this.dropdownEl, 'show');
        addClass(dropdownMenuEl, 'show');
        for (let j = 0, len = toggles.length; j < len; j++) {
            toggles[j].setAttribute('aria-expanded', 'true');
        }
        document.removeEventListener('click', this.handleDropdownClickOutside);
        setTimeout(() => {
            document.removeEventListener('click', this.handleDropdownClickOutside);
            document.addEventListener('click', this.handleDropdownClickOutside);
        }, 0);
        const dropdownMenuTransitionDuration = getTransitionDurationFromElement(dropdownMenuEl);
        this.initPopper(dropdownMenuEl);
        setTimeout(() => {
            customEvent(this.dropdownEl, this.shownEventName);
            // this.shown_bs_dropdown.emit(event);
        }, dropdownMenuTransitionDuration);
    }
    handleHideDropdown() {
        this.show = false;
        customEvent(this.dropdownEl, this.hideEventName);
        // this.hide_bs_dropdown.emit(event);
        const dropdownMenuEl = this.dropdownEl.querySelector('.dropdown-menu');
        const toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        document.removeEventListener('click', this.handleDropdownClickOutside);
        removeClass(this.dropdownEl, 'show');
        removeClass(dropdownMenuEl, 'show');
        for (let j = 0, len = toggles.length; j < len; j++) {
            toggles[j].setAttribute('aria-expanded', 'false');
        }
        const dropdownMenuTransitionDuration = getTransitionDurationFromElement(dropdownMenuEl);
        setTimeout(() => {
            // this.hidden_bs_dropdown.emit(event);
            customEvent(this.dropdownEl, this.hiddenEventName);
        }, dropdownMenuTransitionDuration);
    }
    toggle() {
        this.handleToggleDropdownOnToggleClick();
    }
    update() {
        this.inNavbar = this.detectNavbar(this.dropdownEl);
        if (this.popperHandle !== null) {
            this.popperHandle.scheduleUpdate();
        }
    }
    showDropdown() {
        this.handleShowDropdown();
    }
    hideDropdown() {
        this.handleHideDropdown();
    }
    setDropdownVisibility(toShow) {
        if (toShow) {
            this.handleShowDropdown();
        }
        else {
            this.handleHideDropdown();
        }
    }
    render() {
        return (h("slot", null));
    }
    static get is() { return "bs-dropdown"; }
    static get properties() { return {
        "boundary": {
            "type": "Any",
            "attr": "boundary"
        },
        "display": {
            "type": String,
            "attr": "display"
        },
        "dropdownEl": {
            "elementRef": true
        },
        "dropdownId": {
            "state": true
        },
        "flip": {
            "type": Boolean,
            "attr": "flip"
        },
        "hiddenEventName": {
            "type": String,
            "attr": "hidden-event-name"
        },
        "hideDropdown": {
            "method": true
        },
        "hideEventName": {
            "type": String,
            "attr": "hide-event-name"
        },
        "inNavbar": {
            "state": true
        },
        "keepOpen": {
            "type": Boolean,
            "attr": "keep-open"
        },
        "offset": {
            "type": "Any",
            "attr": "offset"
        },
        "popperHandle": {
            "state": true
        },
        "reference": {
            "type": "Any",
            "attr": "reference"
        },
        "setDropdownVisibility": {
            "method": true
        },
        "show": {
            "type": Boolean,
            "attr": "show",
            "mutable": true
        },
        "showDropdown": {
            "method": true
        },
        "showEventName": {
            "type": String,
            "attr": "show-event-name"
        },
        "shownEventName": {
            "type": String,
            "attr": "shown-event-name"
        },
        "toggle": {
            "method": true
        },
        "update": {
            "method": true
        }
    }; }
}
