/*! Built with http://stenciljs.com */
import { h } from './bscomponents.core.js';
import { l as hasClass, m as addClass, n as removeClass, o as customEvent } from './chunk-1adf00f9.js';
import { a as closest } from './chunk-b81252fc.js';
import { a as Popper, b as getUniqueId } from './chunk-121f5069.js';
import { f as getTransitionDurationFromElement } from './chunk-f2c13081.js';
import './chunk-e18b632f.js';
var AttachmentMap = {
    TOP: 'top-start',
    TOPEND: 'top-end',
    BOTTOM: 'bottom-start',
    BOTTOMEND: 'bottom-end',
    RIGHT: 'right-start',
    RIGHTEND: 'right-end',
    LEFT: 'left-start',
    LEFTEND: 'left-end'
};
var ClassName = {
    DISABLED: 'disabled',
    SHOW: 'show',
    DROPUP: 'dropup',
    DROPRIGHT: 'dropright',
    DROPLEFT: 'dropleft',
    MENURIGHT: 'dropdown-menu-right',
    MENULEFT: 'dropdown-menu-left',
    POSITION_STATIC: 'position-static'
};
function getPlacement(dropdownEl, dropdownMenuEl) {
    var placement = AttachmentMap.BOTTOM;
    // Handle dropup
    if (hasClass(dropdownEl, ClassName.DROPUP)) {
        placement = AttachmentMap.TOP;
        if (hasClass(dropdownMenuEl, ClassName.DROPUP)) {
            placement = AttachmentMap.TOPEND;
        }
    }
    else if (hasClass(dropdownEl, ClassName.DROPRIGHT)) {
        placement = AttachmentMap.RIGHT;
    }
    else if (hasClass(dropdownEl, ClassName.DROPLEFT)) {
        placement = AttachmentMap.LEFT;
    }
    else if (hasClass(dropdownEl, ClassName.MENURIGHT)) {
        placement = AttachmentMap.BOTTOMEND;
    }
    return placement;
}
function getPopperDropdownConfig(dropdownEl, dropdownMenuEl, popperSettings) {
    var popperConfig = {
        placement: getPlacement(dropdownEl, dropdownMenuEl),
        modifiers: {
            offset: popperSettings.offset,
            flip: {
                enabled: popperSettings.flip,
            },
            preventOverflow: {
                boundariesElement: popperSettings.boundary,
            }
        }
    };
    // Disable Popper.js if we have a static display
    if (popperSettings.display === 'static') {
        popperConfig.modifiers.applyStyle = {
            enabled: false
        };
    }
    return popperConfig;
}
function clickWasInside(el, selector) {
    return (closest(el, selector) !== null);
}
// TODO:
// data-offset="10,20"
// data-reference="parent"
// popper.js stuff:
// offset
// flip
// boundary
// reference
// display
var BsDropdown = /** @class */ (function () {
    function BsDropdown() {
        var _this = this;
        this.show = false;
        this.keepOpen = false;
        this.offset = 0;
        this.flip = true;
        this.boundary = 'scrollParent';
        this.reference = 'toggle';
        this.display = 'dynamic';
        this.showEventName = 'show.bs.dropdown';
        this.shownEventName = 'shown.bs.dropdown';
        this.hideEventName = 'hide.bs.dropdown';
        this.hiddenEventName = 'hidden.bs.dropdown';
        this.handleDropdownClickOutside = function (event) {
            // https://gist.github.com/Restuta/e400a555ba24daa396cc
            // handleDropdownClickOutside is an arrow function so that removeEventListener will work
            if (!_this.show) {
                return;
            }
            var clickWasInsideDropdown = clickWasInside(event.target, "[data-bs-id=\"" + _this.dropdownId + "\"]");
            if (!clickWasInsideDropdown || !_this.keepOpen) {
                document.removeEventListener('click', _this.handleDropdownClickOutside);
                _this.show = true;
                _this.handleHideDropdown();
            }
        };
        this.handleToggleDropdownOnToggleClick = function () {
            if (_this.show === true) {
                _this.handleHideDropdown();
            }
            else {
                _this.handleShowDropdown();
            }
        };
    }
    // @Event() show_bs_dropdown: EventEmitter;
    // @Event() shown_bs_dropdown: EventEmitter;
    // @Event() hide_bs_dropdown: EventEmitter;
    // @Event() hidden_bs_dropdown: EventEmitter;
    BsDropdown.prototype.componentWillLoad = function () {
        var _this = this;
        if (hasClass(this.dropdownEl, 'show') && this.show === false) {
            this.show = true;
        }
        if (this.show === true) {
            setTimeout(function () {
                document.removeEventListener('click', _this.handleDropdownClickOutside);
                document.addEventListener('click', _this.handleDropdownClickOutside);
            }, 0);
        }
        this.dropdownId = getUniqueId('dropdown');
        this.dropdownEl.dataset.bsId = this.dropdownId;
        // this.dropdownEl.setAttribute('data-bs-id', this.dropdownId);
        var toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        for (var j = 0, len = toggles.length; j < len; j++) {
            toggles[j].removeEventListener('click', this.handleToggleDropdownOnToggleClick);
            toggles[j].addEventListener('click', this.handleToggleDropdownOnToggleClick);
        }
    };
    BsDropdown.prototype.componentDidUnload = function () {
        document.removeEventListener('click', this.handleDropdownClickOutside);
        var toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        for (var j = 0, len = toggles.length; j < len; j++) {
            toggles[j].removeEventListener('click', this.handleToggleDropdownOnToggleClick);
        }
        if (this.popperHandle !== null) {
            this.popperHandle.destroy();
            this.popperHandle = null;
        }
    };
    BsDropdown.prototype.initPopper = function (dropdownMenuEl) {
        this.inNavbar = this.detectNavbar(this.dropdownEl);
        if (!this.inNavbar) {
            var popperSettings = {
                offset: this.offset,
                flip: this.flip,
                boundary: this.boundary,
                display: this.display,
            };
            var popperConfig = getPopperDropdownConfig(this.dropdownEl, dropdownMenuEl, popperSettings);
            // If boundary is not `scrollParent`, then set position to `static`
            // to allow the menu to "escape" the scroll parent's boundaries
            // https://github.com/twbs/bootstrap/issues/24251
            if (this.boundary !== 'scrollParent') {
                addClass(this.dropdownEl, 'position-static');
            }
            var referenceElement = this.dropdownEl;
            if (this.reference === 'parent') {
                referenceElement = this.dropdownEl.parentNode;
            }
            else if (this.reference instanceof Element) {
                referenceElement = this.reference;
            }
            this.popperHandle = new Popper(referenceElement, dropdownMenuEl, popperConfig);
        }
    };
    BsDropdown.prototype.detectNavbar = function (el) {
        return closest(el, '.navbar') !== null;
    };
    BsDropdown.prototype.handleShowDropdown = function () {
        var _this = this;
        this.show = true;
        // this.show_bs_dropdown.emit(event);
        customEvent(this.dropdownEl, this.showEventName);
        var dropdownMenuEl = this.dropdownEl.querySelector('.dropdown-menu');
        var toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        addClass(this.dropdownEl, 'show');
        addClass(dropdownMenuEl, 'show');
        for (var j = 0, len = toggles.length; j < len; j++) {
            toggles[j].setAttribute('aria-expanded', 'true');
        }
        document.removeEventListener('click', this.handleDropdownClickOutside);
        setTimeout(function () {
            document.removeEventListener('click', _this.handleDropdownClickOutside);
            document.addEventListener('click', _this.handleDropdownClickOutside);
        }, 0);
        var dropdownMenuTransitionDuration = getTransitionDurationFromElement(dropdownMenuEl);
        this.initPopper(dropdownMenuEl);
        setTimeout(function () {
            customEvent(_this.dropdownEl, _this.shownEventName);
            // this.shown_bs_dropdown.emit(event);
        }, dropdownMenuTransitionDuration);
    };
    BsDropdown.prototype.handleHideDropdown = function () {
        var _this = this;
        this.show = false;
        customEvent(this.dropdownEl, this.hideEventName);
        // this.hide_bs_dropdown.emit(event);
        var dropdownMenuEl = this.dropdownEl.querySelector('.dropdown-menu');
        var toggles = this.dropdownEl.querySelectorAll('[data-toggle="dropdown"]');
        document.removeEventListener('click', this.handleDropdownClickOutside);
        removeClass(this.dropdownEl, 'show');
        removeClass(dropdownMenuEl, 'show');
        for (var j = 0, len = toggles.length; j < len; j++) {
            toggles[j].setAttribute('aria-expanded', 'false');
        }
        var dropdownMenuTransitionDuration = getTransitionDurationFromElement(dropdownMenuEl);
        setTimeout(function () {
            // this.hidden_bs_dropdown.emit(event);
            customEvent(_this.dropdownEl, _this.hiddenEventName);
        }, dropdownMenuTransitionDuration);
    };
    BsDropdown.prototype.toggle = function () {
        this.handleToggleDropdownOnToggleClick();
    };
    BsDropdown.prototype.update = function () {
        this.inNavbar = this.detectNavbar(this.dropdownEl);
        if (this.popperHandle !== null) {
            this.popperHandle.scheduleUpdate();
        }
    };
    BsDropdown.prototype.showDropdown = function () {
        this.handleShowDropdown();
    };
    BsDropdown.prototype.hideDropdown = function () {
        this.handleHideDropdown();
    };
    BsDropdown.prototype.setDropdownVisibility = function (toShow) {
        if (toShow) {
            this.handleShowDropdown();
        }
        else {
            this.handleHideDropdown();
        }
    };
    BsDropdown.prototype.render = function () {
        return (h("slot", null));
    };
    Object.defineProperty(BsDropdown, "is", {
        get: function () { return "bs-dropdown"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsDropdown, "properties", {
        get: function () {
            return {
                "boundary": {
                    "type": "Any",
                    "attr": "boundary"
                },
                "display": {
                    "type": String,
                    "attr": "display"
                },
                "dropdownEl": {
                    "elementRef": true
                },
                "dropdownId": {
                    "state": true
                },
                "flip": {
                    "type": Boolean,
                    "attr": "flip"
                },
                "hiddenEventName": {
                    "type": String,
                    "attr": "hidden-event-name"
                },
                "hideDropdown": {
                    "method": true
                },
                "hideEventName": {
                    "type": String,
                    "attr": "hide-event-name"
                },
                "inNavbar": {
                    "state": true
                },
                "keepOpen": {
                    "type": Boolean,
                    "attr": "keep-open"
                },
                "offset": {
                    "type": "Any",
                    "attr": "offset"
                },
                "popperHandle": {
                    "state": true
                },
                "reference": {
                    "type": "Any",
                    "attr": "reference"
                },
                "setDropdownVisibility": {
                    "method": true
                },
                "show": {
                    "type": Boolean,
                    "attr": "show",
                    "mutable": true
                },
                "showDropdown": {
                    "method": true
                },
                "showEventName": {
                    "type": String,
                    "attr": "show-event-name"
                },
                "shownEventName": {
                    "type": String,
                    "attr": "shown-event-name"
                },
                "toggle": {
                    "method": true
                },
                "update": {
                    "method": true
                }
            };
        },
        enumerable: true,
        configurable: true
    });
    return BsDropdown;
}());
export { BsDropdown };
