/*! Built with http://stenciljs.com */
import { h } from './bscomponents.core.js';
import { i as toNumber_1, j as baseIndexOf, g as baseUnary, b as identity, k as split, e as toLower, l as isNaN$1, m as _isNumber, f as getTransitionDurationFromElement } from './chunk-f2c13081.js';
import { w as arrayMap, f as isArrayLike, d as isObjectLike, v as toString, l as hasClass, n as removeClass, m as addClass, o as customEvent } from './chunk-1adf00f9.js';
import { a as keys, b as SetCache, c as cacheHas } from './chunk-20645c8e.js';
import { e as isString, a as _size } from './chunk-10f3ce4f.js';
import { a as getNative, d as _get } from './chunk-def07e68.js';
import { a as Popper, b as getUniqueId } from './chunk-121f5069.js';
import { b as _has } from './chunk-11f8e35c.js';
import { a as closest } from './chunk-b81252fc.js';
import { a as getConfigBoolean } from './chunk-4ac9287e.js';
import './chunk-e18b632f.js';
/** Used as references for various `Number` constants. */
var INFINITY = 1 / 0, MAX_INTEGER = 1.7976931348623157e+308;
/**
 * Converts `value` to a finite number.
 *
 * @static
 * @memberOf _
 * @since 4.12.0
 * @category Lang
 * @param {*} value The value to convert.
 * @returns {number} Returns the converted number.
 * @example
 *
 * _.toFinite(3.2);
 * // => 3.2
 *
 * _.toFinite(Number.MIN_VALUE);
 * // => 5e-324
 *
 * _.toFinite(Infinity);
 * // => 1.7976931348623157e+308
 *
 * _.toFinite('3.2');
 * // => 3.2
 */
function toFinite(value) {
    if (!value) {
        return value === 0 ? value : 0;
    }
    value = toNumber_1(value);
    if (value === INFINITY || value === -INFINITY) {
        var sign = (value < 0 ? -1 : 1);
        return sign * MAX_INTEGER;
    }
    return value === value ? value : 0;
}
var toFinite_1 = toFinite;
/**
 * Converts `value` to an integer.
 *
 * **Note:** This method is loosely based on
 * [`ToInteger`](http://www.ecma-international.org/ecma-262/7.0/#sec-tointeger).
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to convert.
 * @returns {number} Returns the converted integer.
 * @example
 *
 * _.toInteger(3.2);
 * // => 3
 *
 * _.toInteger(Number.MIN_VALUE);
 * // => 0
 *
 * _.toInteger(Infinity);
 * // => 1.7976931348623157e+308
 *
 * _.toInteger('3.2');
 * // => 3
 */
function toInteger(value) {
    var result = toFinite_1(value), remainder = result % 1;
    return result === result ? (remainder ? result - remainder : result) : 0;
}
var toInteger_1 = toInteger;
/**
 * The base implementation of `_.values` and `_.valuesIn` which creates an
 * array of `object` property values corresponding to the property names
 * of `props`.
 *
 * @private
 * @param {Object} object The object to query.
 * @param {Array} props The property names to get values for.
 * @returns {Object} Returns the array of property values.
 */
function baseValues(object, props) {
    return arrayMap(props, function (key) {
        return object[key];
    });
}
var _baseValues = baseValues;
/**
 * Creates an array of the own enumerable string keyed property values of `object`.
 *
 * **Note:** Non-object values are coerced to objects.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Object
 * @param {Object} object The object to query.
 * @returns {Array} Returns the array of property values.
 * @example
 *
 * function Foo() {
 *   this.a = 1;
 *   this.b = 2;
 * }
 *
 * Foo.prototype.c = 3;
 *
 * _.values(new Foo);
 * // => [1, 2] (iteration order is not guaranteed)
 *
 * _.values('hi');
 * // => ['h', 'i']
 */
function values(object) {
    return object == null ? [] : _baseValues(object, keys(object));
}
var values_1 = values;
/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeMax = Math.max;
/**
 * Checks if `value` is in `collection`. If `collection` is a string, it's
 * checked for a substring of `value`, otherwise
 * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
 * is used for equality comparisons. If `fromIndex` is negative, it's used as
 * the offset from the end of `collection`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Collection
 * @param {Array|Object|string} collection The collection to inspect.
 * @param {*} value The value to search for.
 * @param {number} [fromIndex=0] The index to search from.
 * @param- {Object} [guard] Enables use as an iteratee for methods like `_.reduce`.
 * @returns {boolean} Returns `true` if `value` is found, else `false`.
 * @example
 *
 * _.includes([1, 2, 3], 1);
 * // => true
 *
 * _.includes([1, 2, 3], 1, 2);
 * // => false
 *
 * _.includes({ 'a': 1, 'b': 2 }, 1);
 * // => true
 *
 * _.includes('abcd', 'bc');
 * // => true
 */
function includes(collection, value, fromIndex, guard) {
    collection = isArrayLike(collection) ? collection : values_1(collection);
    fromIndex = (fromIndex && !guard) ? toInteger_1(fromIndex) : 0;
    var length = collection.length;
    if (fromIndex < 0) {
        fromIndex = nativeMax(length + fromIndex, 0);
    }
    return isString(collection)
        ? (fromIndex <= length && collection.indexOf(value, fromIndex) > -1)
        : (!!length && baseIndexOf(collection, value, fromIndex) > -1);
}
var includes_1 = includes;
/**
 * A specialized version of `_.includes` for arrays without support for
 * specifying an index to search from.
 *
 * @private
 * @param {Array} [array] The array to inspect.
 * @param {*} target The value to search for.
 * @returns {boolean} Returns `true` if `target` is found, else `false`.
 */
function arrayIncludes(array, value) {
    var length = array == null ? 0 : array.length;
    return !!length && baseIndexOf(array, value, 0) > -1;
}
var _arrayIncludes = arrayIncludes;
/**
 * This function is like `arrayIncludes` except that it accepts a comparator.
 *
 * @private
 * @param {Array} [array] The array to inspect.
 * @param {*} target The value to search for.
 * @param {Function} comparator The comparator invoked per element.
 * @returns {boolean} Returns `true` if `target` is found, else `false`.
 */
function arrayIncludesWith(array, value, comparator) {
    var index = -1, length = array == null ? 0 : array.length;
    while (++index < length) {
        if (comparator(value, array[index])) {
            return true;
        }
    }
    return false;
}
var _arrayIncludesWith = arrayIncludesWith;
/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeMin = Math.min;
/**
 * The base implementation of methods like `_.intersection`, without support
 * for iteratee shorthands, that accepts an array of arrays to inspect.
 *
 * @private
 * @param {Array} arrays The arrays to inspect.
 * @param {Function} [iteratee] The iteratee invoked per element.
 * @param {Function} [comparator] The comparator invoked per element.
 * @returns {Array} Returns the new array of shared values.
 */
function baseIntersection(arrays, iteratee, comparator) {
    var includes = comparator ? _arrayIncludesWith : _arrayIncludes, length = arrays[0].length, othLength = arrays.length, othIndex = othLength, caches = Array(othLength), maxLength = Infinity, result = [];
    while (othIndex--) {
        var array = arrays[othIndex];
        if (othIndex && iteratee) {
            array = arrayMap(array, baseUnary(iteratee));
        }
        maxLength = nativeMin(array.length, maxLength);
        caches[othIndex] = !comparator && (iteratee || (length >= 120 && array.length >= 120))
            ? new SetCache(othIndex && array)
            : undefined;
    }
    array = arrays[0];
    var index = -1, seen = caches[0];
    outer: while (++index < length && result.length < maxLength) {
        var value = array[index], computed = iteratee ? iteratee(value) : value;
        value = (comparator || value !== 0) ? value : 0;
        if (!(seen
            ? cacheHas(seen, computed)
            : includes(result, computed, comparator))) {
            othIndex = othLength;
            while (--othIndex) {
                var cache = caches[othIndex];
                if (!(cache
                    ? cacheHas(cache, computed)
                    : includes(arrays[othIndex], computed, comparator))) {
                    continue outer;
                }
            }
            if (seen) {
                seen.push(computed);
            }
            result.push(value);
        }
    }
    return result;
}
var _baseIntersection = baseIntersection;
/**
 * A faster alternative to `Function#apply`, this function invokes `func`
 * with the `this` binding of `thisArg` and the arguments of `args`.
 *
 * @private
 * @param {Function} func The function to invoke.
 * @param {*} thisArg The `this` binding of `func`.
 * @param {Array} args The arguments to invoke `func` with.
 * @returns {*} Returns the result of `func`.
 */
function apply(func, thisArg, args) {
    switch (args.length) {
        case 0: return func.call(thisArg);
        case 1: return func.call(thisArg, args[0]);
        case 2: return func.call(thisArg, args[0], args[1]);
        case 3: return func.call(thisArg, args[0], args[1], args[2]);
    }
    return func.apply(thisArg, args);
}
var _apply = apply;
/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeMax$1 = Math.max;
/**
 * A specialized version of `baseRest` which transforms the rest array.
 *
 * @private
 * @param {Function} func The function to apply a rest parameter to.
 * @param {number} [start=func.length-1] The start position of the rest parameter.
 * @param {Function} transform The rest array transform.
 * @returns {Function} Returns the new function.
 */
function overRest(func, start, transform) {
    start = nativeMax$1(start === undefined ? (func.length - 1) : start, 0);
    return function () {
        var args = arguments, index = -1, length = nativeMax$1(args.length - start, 0), array = Array(length);
        while (++index < length) {
            array[index] = args[start + index];
        }
        index = -1;
        var otherArgs = Array(start + 1);
        while (++index < start) {
            otherArgs[index] = args[index];
        }
        otherArgs[start] = transform(array);
        return _apply(func, this, otherArgs);
    };
}
var _overRest = overRest;
/**
 * Creates a function that returns `value`.
 *
 * @static
 * @memberOf _
 * @since 2.4.0
 * @category Util
 * @param {*} value The value to return from the new function.
 * @returns {Function} Returns the new constant function.
 * @example
 *
 * var objects = _.times(2, _.constant({ 'a': 1 }));
 *
 * console.log(objects);
 * // => [{ 'a': 1 }, { 'a': 1 }]
 *
 * console.log(objects[0] === objects[1]);
 * // => true
 */
function constant(value) {
    return function () {
        return value;
    };
}
var constant_1 = constant;
var defineProperty = (function () {
    try {
        var func = getNative(Object, 'defineProperty');
        func({}, '', {});
        return func;
    }
    catch (e) { }
}());
var _defineProperty = defineProperty;
/**
 * The base implementation of `setToString` without support for hot loop shorting.
 *
 * @private
 * @param {Function} func The function to modify.
 * @param {Function} string The `toString` result.
 * @returns {Function} Returns `func`.
 */
var baseSetToString = !_defineProperty ? identity : function (func, string) {
    return _defineProperty(func, 'toString', {
        'configurable': true,
        'enumerable': false,
        'value': constant_1(string),
        'writable': true
    });
};
var _baseSetToString = baseSetToString;
/** Used to detect hot functions by number of calls within a span of milliseconds. */
var HOT_COUNT = 800, HOT_SPAN = 16;
/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeNow = Date.now;
/**
 * Creates a function that'll short out and invoke `identity` instead
 * of `func` when it's called `HOT_COUNT` or more times in `HOT_SPAN`
 * milliseconds.
 *
 * @private
 * @param {Function} func The function to restrict.
 * @returns {Function} Returns the new shortable function.
 */
function shortOut(func) {
    var count = 0, lastCalled = 0;
    return function () {
        var stamp = nativeNow(), remaining = HOT_SPAN - (stamp - lastCalled);
        lastCalled = stamp;
        if (remaining > 0) {
            if (++count >= HOT_COUNT) {
                return arguments[0];
            }
        }
        else {
            count = 0;
        }
        return func.apply(undefined, arguments);
    };
}
var _shortOut = shortOut;
/**
 * Sets the `toString` method of `func` to return `string`.
 *
 * @private
 * @param {Function} func The function to modify.
 * @param {Function} string The `toString` result.
 * @returns {Function} Returns `func`.
 */
var setToString = _shortOut(_baseSetToString);
var _setToString = setToString;
/**
 * The base implementation of `_.rest` which doesn't validate or coerce arguments.
 *
 * @private
 * @param {Function} func The function to apply a rest parameter to.
 * @param {number} [start=func.length-1] The start position of the rest parameter.
 * @returns {Function} Returns the new function.
 */
function baseRest(func, start) {
    return _setToString(_overRest(func, start, identity), func + '');
}
var _baseRest = baseRest;
/**
 * This method is like `_.isArrayLike` except that it also checks if `value`
 * is an object.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an array-like object,
 *  else `false`.
 * @example
 *
 * _.isArrayLikeObject([1, 2, 3]);
 * // => true
 *
 * _.isArrayLikeObject(document.body.children);
 * // => true
 *
 * _.isArrayLikeObject('abc');
 * // => false
 *
 * _.isArrayLikeObject(_.noop);
 * // => false
 */
function isArrayLikeObject(value) {
    return isObjectLike(value) && isArrayLike(value);
}
var isArrayLikeObject_1 = isArrayLikeObject;
/**
 * Casts `value` to an empty array if it's not an array like object.
 *
 * @private
 * @param {*} value The value to inspect.
 * @returns {Array|Object} Returns the cast array-like object.
 */
function castArrayLikeObject(value) {
    return isArrayLikeObject_1(value) ? value : [];
}
var _castArrayLikeObject = castArrayLikeObject;
/**
 * Creates an array of unique values that are included in all given arrays
 * using [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
 * for equality comparisons. The order and references of result values are
 * determined by the first array.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Array
 * @param {...Array} [arrays] The arrays to inspect.
 * @returns {Array} Returns the new array of intersecting values.
 * @example
 *
 * _.intersection([2, 1], [2, 3]);
 * // => [2]
 */
var intersection = _baseRest(function (arrays) {
    var mapped = arrayMap(arrays, _castArrayLikeObject);
    return (mapped.length && mapped[0] === arrays[0])
        ? _baseIntersection(mapped)
        : [];
});
var intersection_1 = intersection;
function parseJson(str) {
    try {
        return JSON.parse(str);
    }
    catch (e) {
        return false;
    }
}
var BsTooltip = /** @class */ (function () {
    function BsTooltip() {
        var _this = this;
        this.noEnableOnLoad = false;
        this.showEventName = 'show.bs.tooltip';
        this.shownEventName = 'shown.bs.tooltip';
        this.hideEventName = 'hide.bs.tooltip';
        this.hiddenEventName = 'hidden.bs.tooltip';
        this.insertedEventName = 'inserted.bs.tooltip';
        this.handlePopperOnCreate = function (data) {
            if (data.originalPlacement !== data.placement) {
                _this.handlePopperPlacementChange(data);
            }
        };
        this.handlePopperOnUpdate = function (data) {
            _this.handlePopperPlacementChange(data);
        };
        this.handleClickTrigger = function (event) {
            _this.toggle(event);
        };
        this.handleMouseEnter = function (event) {
            _this.enter(event);
        };
        this.handleFocusIn = function (event) {
            _this.enter(event);
        };
        this.handleMouseLeave = function (event) {
            _this.leave(event);
        };
        this.handleFocusOut = function (event) {
            _this.leave(event);
        };
        this.handleModalHide = function (event) {
            _this.activeTrigger.click = false;
            _this.leave(event);
        };
    }
    // TODO: a stencil watch on a prop to update the title on change would be nice. . .
    BsTooltip.prototype.componentDidLoad = function () {
        var currentTabIndex = this.tooltipEl.getAttribute('tabindex');
        if (_size(currentTabIndex) === 0) {
            // without tabindex set the bs-tooltip can not receive focus
            this.tooltipEl.setAttribute('tabindex', '0');
        }
        if (!this.noEnableOnLoad && !this.isEnabled) {
            this.enableTooltip();
        }
    };
    BsTooltip.prototype.componentDidUnload = function () {
        this.disableTooltip();
    };
    BsTooltip.prototype.enableTooltip = function (overrideConfig) {
        if (overrideConfig === void 0) { overrideConfig = {}; }
        this.isEnabled = true;
        this.activeTrigger = {};
        this.hoverState = '';
        this.tooltipId = getUniqueId('tooltip');
        this.tooltipEl.dataset.bsId = this.tooltipId;
        this.tip = null;
        this.getConfig(overrideConfig);
        this.setListeners();
    };
    BsTooltip.prototype.makeTip = function () {
        var container = this.config.container === false ? document.body : document.querySelector(this.config.container);
        var tip = document.createElement("div");
        addClass(tip, 'tooltip');
        tip.setAttribute('role', 'tooltip');
        tip.setAttribute('id', this.tooltipId);
        this.tooltipEl.setAttribute('aria-describedby', this.tooltipId);
        tip.innerHTML = '<div class="arrow"></div>' + '<div class="tooltip-inner"></div>';
        if (!this.tooltipEl.ownerDocument.documentElement.contains(tip)) {
            customEvent(this.tooltipEl, this.insertedEventName);
            container.appendChild(tip);
        }
        return tip;
    };
    BsTooltip.prototype.getTipElement = function () {
        clearTimeout(this.disposeTimeout);
        this.tip = this.tip || this.makeTip();
        return this.tip;
    };
    ;
    BsTooltip.prototype.disableTooltip = function () {
        this.isEnabled = false;
        clearTimeout(this.timeout);
        this.tooltipEl.removeEventListener('click', this.handleClickTrigger);
        this.tooltipEl.removeEventListener('mouseenter', this.handleMouseEnter);
        this.tooltipEl.removeEventListener('mouseleave', this.handleMouseLeave);
        this.tooltipEl.removeEventListener('focusin', this.handleFocusIn);
        this.tooltipEl.removeEventListener('focusout', this.handleFocusOut);
        var closestModal = closest(this.tooltipEl, '.modal');
        if (closestModal) {
            var modalHideEventName = closestModal.hideEventName;
            if (modalHideEventName) {
                closestModal.removeEventListener(closestModal.hideEventName, this.handleModalHide);
            }
        }
        this.activeTrigger = {};
        this.hoverState = '';
        clearTimeout(this.disposeTimeout);
        this.removeTooltipFromDom();
    };
    BsTooltip.prototype.toggle = function (event) {
        if (event === void 0) { event = null; }
        if (!this.isEnabled) {
            return;
        }
        if (event) {
            this.activeTrigger.click = !this.activeTrigger.click;
            if (this.isWithActiveTrigger()) {
                this.enter();
            }
            else {
                this.leave();
            }
        }
        else {
            if (hasClass(this.getTipElement(), 'show')) {
                this.leave();
            }
            else {
                this.enter();
            }
        }
    };
    BsTooltip.prototype.isWithActiveTrigger = function () {
        for (var trigger in this.activeTrigger) {
            if (this.activeTrigger[trigger]) {
                return true;
            }
        }
        return false;
    };
    ;
    BsTooltip.prototype.enter = function (event) {
        var _this = this;
        if (event === void 0) { event = null; }
        if (event) {
            var eventType = event.type === 'focusin' ? 'focus' : 'hover';
            this.activeTrigger[eventType] = true;
        }
        if (hasClass(this.getTipElement(), 'show') || this.hoverState === 'show') {
            this.hoverState = 'show';
            return;
        }
        clearTimeout(this.timeout);
        this.hoverState = 'show';
        if (!this.config.delay || !this.config.delay.show) {
            this.show();
            return;
        }
        this.timeout = setTimeout(function () {
            if (_this.hoverState === 'show') {
                _this.show();
            }
        }, this.config.delay.show);
    };
    BsTooltip.prototype.leave = function (event) {
        var _this = this;
        if (event === void 0) { event = null; }
        // console.log('leave');
        if (event) {
            var eventType = event.type === 'focusout' ? 'focus' : 'hover';
            this.activeTrigger[eventType] = false;
        }
        if (this.isWithActiveTrigger()) {
            return;
        }
        clearTimeout(this.timeout);
        this.hoverState = 'out';
        if (!this.config.delay || !this.config.delay.hide) {
            this.hide();
            return;
        }
        this.timeout = setTimeout(function () {
            if (_this.hoverState === 'out') {
                _this.hide();
            }
        }, this.config.delay.hide);
    };
    BsTooltip.prototype.show = function () {
        var _this = this;
        if (this.tooltipEl.style.display === 'none') {
            throw new Error('Please use show on visible elements');
        }
        if (_size(this.config.title) > 0 && this.isEnabled) {
            var showEvent = customEvent(this.tooltipEl, this.showEventName);
            var isInTheDom = this.tooltipEl.ownerDocument.documentElement.contains(this.tooltipEl);
            if (showEvent.defaultPrevented || !isInTheDom) {
                return;
            }
            var tip = this.getTipElement();
            this.setContent();
            if (this.config.animation) {
                addClass(tip, 'fade');
            }
            var placement = typeof this.config.placement === 'function' ? this.config.placement.call(this, tip, this.tooltipEl) : this.config.placement;
            var attachment = this.getAttachment(placement);
            this.addAttachmentClass(attachment);
            this.popperHandle = new Popper(this.tooltipEl, tip, {
                placement: attachment,
                modifiers: {
                    offset: {
                        offset: this.config.offset
                    },
                    flip: {
                        behavior: this.config.fallbackPlacement
                    },
                    arrow: {
                        element: '.arrow',
                    },
                    preventOverflow: {
                        boundariesElement: this.config.boundary
                    }
                },
                onCreate: this.handlePopperOnCreate,
                onUpdate: this.handlePopperOnUpdate,
            });
            addClass(tip, 'show');
            // If this is a touch-enabled device we add extra
            // empty mouseover listeners to the body's immediate children;
            // only needed because of broken event delegation on iOS
            // https://www.quirksmode.org/blog/archives/2014/02/mouse_event_bub.html
            // if ('ontouchstart' in document.documentElement) {
            //   $$$1(document.body).children().on('mouseover', null, $$$1.noop);
            // }
            if (hasClass(tip, 'fade')) {
                var transitionDuration = getTransitionDurationFromElement(tip);
                setTimeout(function () {
                    _this.showComplete();
                }, transitionDuration);
            }
            else {
                this.showComplete();
            }
        }
    };
    BsTooltip.prototype.showComplete = function () {
        if (this.config.animation) {
            this.fixTransition();
        }
        var prevHoverState = this.hoverState;
        this.hoverState = null;
        customEvent(this.tooltipEl, this.shownEventName);
        if (prevHoverState === 'out') {
            this.leave();
        }
    };
    BsTooltip.prototype.hide = function (callback) {
        var _this = this;
        if (callback === void 0) { callback = function () { }; }
        var tip = this.getTipElement();
        var hideEvent = customEvent(this.tooltipEl, this.hideEventName);
        if (hideEvent.defaultPrevented) {
            return;
        }
        removeClass(tip, 'show');
        // If this is a touch-enabled device we remove the extra
        // empty mouseover listeners we added for iOS support
        // if ('ontouchstart' in document.documentElement) {
        //   $$$1(document.body).children().off('mouseover', null, $$$1.noop);
        // }
        this.activeTrigger.click = false;
        this.activeTrigger.focus = false;
        this.activeTrigger.hover = false;
        if (hasClass(tip, 'fade')) {
            var transitionDuration = getTransitionDurationFromElement(tip);
            setTimeout(function () {
                _this.hideComplete(callback);
            }, transitionDuration);
        }
        else {
            this.hideComplete();
        }
        this.hoverState = '';
    };
    BsTooltip.prototype.hideComplete = function (callback) {
        var _this = this;
        if (callback === void 0) { callback = function () { }; }
        this.cleanTipClass();
        this.tooltipEl.removeAttribute('aria-describedby');
        customEvent(this.tooltipEl, this.hiddenEventName);
        if (this.popperHandle !== null) {
            this.popperHandle.destroy();
        }
        this.disposeTimeout = setTimeout(function () {
            _this.removeTooltipFromDom();
        }, this.config.disposeTimeToWait);
        callback();
    };
    ;
    BsTooltip.prototype.removeTooltipFromDom = function () {
        if (this.tip && this.hoverState !== 'show' && this.tip.parentNode) {
            this.tip.parentNode.removeChild(this.tip);
            this.tip = null;
        }
    };
    BsTooltip.prototype.fixTransition = function () {
        var tip = this.getTipElement();
        var initConfigAnimation = this.config.animation;
        if (tip.getAttribute('x-placement') !== null) {
            return;
        }
        removeClass(tip, 'fade');
        this.config.animation = false;
        this.hide();
        this.show();
        this.config.animation = initConfigAnimation;
    };
    BsTooltip.prototype.handlePopperPlacementChange = function (popperData) {
        var popperInstance = popperData.instance;
        this.tip = popperInstance.popper;
        this.cleanTipClass();
        this.addAttachmentClass(this.getAttachment(popperData.placement));
    };
    ;
    BsTooltip.prototype.cleanTipClass = function () {
        var tip = this.getTipElement();
        var bsTooltipPositionClasses = [
            'bs-tooltip-auto',
            'bs-tooltip-top',
            'bs-tooltip-right',
            'bs-tooltip-bottom',
            'bs-tooltip-left'
        ];
        var classesToRemove = intersection_1(bsTooltipPositionClasses, tip.classList);
        for (var j = 0, len = classesToRemove.length; j < len; j++) {
            removeClass(tip, classesToRemove[j]);
        }
    };
    ;
    BsTooltip.prototype.addAttachmentClass = function (attachment) {
        var CLASS_PREFIX = 'bs-tooltip';
        addClass(this.getTipElement(), CLASS_PREFIX + "-" + attachment);
    };
    ;
    BsTooltip.prototype.getAttachment = function (placement) {
        var AttachmentMap = {
            auto: 'auto',
            top: 'top',
            right: 'right',
            bottom: 'bottom',
            left: 'left'
        };
        return AttachmentMap[toLower(placement)];
    };
    ;
    BsTooltip.prototype.setContent = function () {
        var tip = this.getTipElement();
        this.setElementContent(tip.querySelector('.tooltip-inner'), this.getTitle());
        removeClass(tip, 'fade');
        removeClass(tip, 'show');
    };
    ;
    BsTooltip.prototype.setElementContent = function (el, content) {
        if (this.config.html) {
            el.innerHTML = content;
        }
        else {
            el.textContent = content;
        }
    };
    ;
    BsTooltip.prototype.getTitle = function () {
        var title = this.tooltipEl.dataset.originalTitle;
        if (!title) {
            title = typeof this.config.title === 'function' ? this.config.title.call(this.tooltipEl) : this.config.title;
        }
        return title;
    };
    ;
    BsTooltip.prototype.setListeners = function () {
        var triggers = split(toLower(this.config.trigger), ' ');
        if (includes_1(triggers, 'click')) {
            this.tooltipEl.removeEventListener('click', this.handleClickTrigger);
            this.tooltipEl.addEventListener('click', this.handleClickTrigger);
        }
        if (includes_1(triggers, 'manual')) {
            // hover and focus events are ignored if manual is included.
            return;
        }
        if (includes_1(triggers, 'hover')) {
            this.tooltipEl.removeEventListener('mouseenter', this.handleMouseEnter);
            this.tooltipEl.addEventListener('mouseenter', this.handleMouseEnter);
            this.tooltipEl.removeEventListener('mouseleave', this.handleMouseLeave);
            this.tooltipEl.addEventListener('mouseleave', this.handleMouseLeave);
        }
        if (includes_1(triggers, 'focus')) {
            this.tooltipEl.removeEventListener('focusin', this.handleFocusIn);
            this.tooltipEl.addEventListener('focusin', this.handleFocusIn);
            this.tooltipEl.removeEventListener('focusout', this.handleFocusOut);
            this.tooltipEl.addEventListener('focusout', this.handleFocusOut);
        }
        var closestModal = closest(this.tooltipEl, '.modal');
        if (closestModal) {
            var modalHideEventName = closestModal.hideEventName;
            // console.log('closestModal: ', closestModal);
            // console.log('closestModal.hideEventName: ', closestModal.hideEventName);
            if (modalHideEventName) {
                closestModal.removeEventListener(closestModal.hideEventName, this.handleModalHide);
                closestModal.addEventListener(closestModal.hideEventName, this.handleModalHide);
            }
        }
        // TODO: what if the selector is set
        // https://github.com/twbs/bootstrap/issues/4215
        // well if it's a dynamic html element. . .
        // we can just add and remove bs-tooltip tags as needed. . .
        this.fixTitle();
    };
    BsTooltip.prototype.fixTitle = function () {
        var titleType = typeof this.tooltipEl.dataset.originalTitle;
        if (this.tooltipEl.title || titleType !== 'string') {
            this.tooltipEl.dataset.originalTitle = this.tooltipEl.title || '';
            this.tooltipEl.title = '';
        }
    };
    BsTooltip.prototype.getConfig = function (overrideConfig) {
        if (overrideConfig === void 0) { overrideConfig = {}; }
        var defaultConfig = {
            animation: true,
            template: '<div class="tooltip" role="tooltip">' + '<div class="arrow"></div>' + '<div class="tooltip-inner"></div></div>',
            trigger: 'hover focus',
            title: '',
            delay: 0,
            html: false,
            selector: false,
            placement: 'top',
            offset: 0,
            container: false,
            fallbackPlacement: 'flip',
            boundary: 'scrollParent',
            disposeTimeToWait: 5000,
        };
        this.config = {};
        var config = {};
        if (_has(overrideConfig, 'animation')) {
            config.animation = getConfigBoolean(overrideConfig.animation);
        }
        else if (_has(this.tooltipEl.dataset, 'animation')) {
            config.animation = getConfigBoolean(this.tooltipEl.dataset.animation);
        }
        else {
            config.animation = defaultConfig.animation;
        }
        if (_has(overrideConfig, 'trigger')) {
            config.trigger = overrideConfig.trigger;
        }
        else if (_has(this.tooltipEl.dataset, 'trigger')) {
            config.trigger = this.tooltipEl.dataset.trigger;
        }
        else {
            config.trigger = defaultConfig.trigger;
        }
        var titleAttribute = this.tooltipEl.getAttribute('title');
        if (_has(overrideConfig, 'title')) {
            config.title = overrideConfig.title;
        }
        else if (_has(this.tooltipEl.dataset, 'title')) {
            config.title = this.tooltipEl.dataset.title;
        }
        else if (titleAttribute) {
            config.title = titleAttribute;
        }
        else {
            config.title = defaultConfig.title;
        }
        if (_isNumber(config.title)) {
            config.title = toString(config.title);
        }
        var newConfigDelay;
        if (_has(overrideConfig, 'delay')) {
            newConfigDelay = overrideConfig.delay;
        }
        else if (_has(this.tooltipEl.dataset, 'delay')) {
            newConfigDelay = this.tooltipEl.dataset.delay;
        }
        if (_size(newConfigDelay) > 0) {
            var configDelayInteger = toInteger_1(newConfigDelay);
            var configDelayObj = parseJson(newConfigDelay);
            if (!isNaN$1(configDelayInteger)) {
                config.delay = {
                    show: configDelayInteger,
                    hide: configDelayInteger
                };
            }
            else if (configDelayObj !== false) {
                config.delay = {
                    show: _get(configDelayObj, 'show', defaultConfig.delay),
                    hide: _get(configDelayObj, 'hide', defaultConfig.delay),
                };
            }
        }
        else {
            config.delay = {
                show: defaultConfig.delay,
                hide: defaultConfig.delay
            };
        }
        if (_has(overrideConfig, 'html')) {
            config.html = getConfigBoolean(overrideConfig.html);
        }
        else if (_has(this.tooltipEl.dataset, 'html')) {
            config.html = getConfigBoolean(this.tooltipEl.dataset.html);
        }
        else {
            config.html = defaultConfig.html;
        }
        if (_has(overrideConfig, 'selector')) {
            config.selector = overrideConfig.selector;
        }
        else if (_has(this.tooltipEl.dataset, 'selector')) {
            config.selector = this.tooltipEl.dataset.selector;
        }
        else {
            config.selector = defaultConfig.selector;
        }
        if (_has(overrideConfig, 'placement')) {
            config.placement = overrideConfig.placement;
        }
        else if (_has(this.tooltipEl.dataset, 'placement')) {
            config.placement = this.tooltipEl.dataset.placement;
        }
        else {
            config.placement = defaultConfig.placement;
        }
        if (_has(overrideConfig, 'offset')) {
            config.offset = toInteger_1(overrideConfig.offset);
        }
        else if (_has(this.tooltipEl.dataset, 'offset')) {
            config.offset = toInteger_1(this.tooltipEl.dataset.offset);
        }
        else {
            config.offset = defaultConfig.offset;
        }
        if (isNaN$1(config.offset)) {
            config.offset = defaultConfig.offset;
        }
        if (_has(overrideConfig, 'container')) {
            config.container = getConfigBoolean(overrideConfig.container);
        }
        else if (_has(this.tooltipEl.dataset, 'container')) {
            config.container = getConfigBoolean(this.tooltipEl.dataset.container);
        }
        else {
            config.container = defaultConfig.container;
        }
        if (_has(overrideConfig, 'fallbackPlacement')) {
            config.fallbackPlacement = overrideConfig.fallbackPlacement;
        }
        else if (_has(this.tooltipEl.dataset, 'fallbackPlacement')) {
            config.fallbackPlacement = this.tooltipEl.dataset.fallbackPlacement;
        }
        else {
            config.fallbackPlacement = defaultConfig.fallbackPlacement;
        }
        if (_has(overrideConfig, 'boundary')) {
            config.boundary = overrideConfig.boundary;
        }
        else if (_has(this.tooltipEl.dataset, 'boundary')) {
            config.boundary = this.tooltipEl.dataset.boundary;
        }
        else {
            config.boundary = defaultConfig.boundary;
        }
        if (_has(overrideConfig, 'disposeTimeToWait')) {
            config.disposeTimeToWait = toInteger_1(overrideConfig.disposeTimeToWait);
        }
        else if (_has(this.tooltipEl.dataset, 'disposeTimeToWait')) {
            config.disposeTimeToWait = toInteger_1(this.tooltipEl.dataset.disposeTimeToWait);
        }
        else {
            config.disposeTimeToWait = defaultConfig.disposeTimeToWait;
        }
        if (isNaN$1(config.disposeTimeToWait)) {
            config.disposeTimeToWait = defaultConfig.disposeTimeToWait;
        }
        this.config = config;
    };
    ;
    BsTooltip.prototype.tooltip = function (tooltipOptions) {
        if (tooltipOptions === void 0) { tooltipOptions = {}; }
        if (_size(tooltipOptions) === 0) {
            if (!this.isEnabled) {
                this.enableTooltip();
            }
        }
        else if (tooltipOptions === 'enable') {
            this.enableTooltip();
        }
        else if (tooltipOptions === 'disable') {
            this.disableTooltip();
        }
        else if (tooltipOptions === 'toggleEnabled') {
            if (this.isEnabled) {
                this.disableTooltip();
            }
            else {
                this.enableTooltip();
            }
        }
        else if (tooltipOptions === 'show') {
            if (!this.isEnabled) {
                this.enableTooltip();
            }
            this.enter();
        }
        else if (tooltipOptions === 'hide') {
            if (!this.isEnabled) {
                this.enableTooltip();
            }
            this.leave();
        }
        else if (tooltipOptions === 'toggle') {
            if (!this.isEnabled) {
                this.enableTooltip();
            }
            this.toggle();
        }
        else if (tooltipOptions === 'update') {
            if (this.popperHandle !== null) {
                this.popperHandle.scheduleUpdate();
            }
        }
        else {
            this.enableTooltip(tooltipOptions);
        }
    };
    BsTooltip.prototype.render = function () {
        return (h("slot", null));
    };
    Object.defineProperty(BsTooltip, "is", {
        get: function () { return "bs-tooltip"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsTooltip, "properties", {
        get: function () {
            return {
                "activeTrigger": {
                    "state": true
                },
                "config": {
                    "state": true
                },
                "disposeTimeout": {
                    "state": true
                },
                "hiddenEventName": {
                    "type": String,
                    "attr": "hidden-event-name"
                },
                "hideEventName": {
                    "type": String,
                    "attr": "hide-event-name"
                },
                "hoverState": {
                    "state": true
                },
                "insertedEventName": {
                    "type": String,
                    "attr": "inserted-event-name"
                },
                "isEnabled": {
                    "state": true
                },
                "noEnableOnLoad": {
                    "type": Boolean,
                    "attr": "no-enable-on-load"
                },
                "popperHandle": {
                    "state": true
                },
                "showEventName": {
                    "type": String,
                    "attr": "show-event-name"
                },
                "shownEventName": {
                    "type": String,
                    "attr": "shown-event-name"
                },
                "timeout": {
                    "state": true
                },
                "tip": {
                    "state": true
                },
                "tooltip": {
                    "method": true
                },
                "tooltipEl": {
                    "elementRef": true
                },
                "tooltipId": {
                    "state": true
                }
            };
        },
        enumerable: true,
        configurable: true
    });
    return BsTooltip;
}());
export { BsTooltip };
